@@ -79,10 +79,10 @@ def test_str_not_json(input_value, expected):
         ({'pattern': r'\d+$'}, 'foobar 123', 'foobar 123'),
         ({'pattern': r'^\d+$'}, '12345a', Err("String should match pattern '^\\d+$' [kind=str_pattern_mismatch")),
         # strip comes after length check
-        ({'max_length': 5, 'strip_whitespace': True}, '1234  ', Err('String should have at most 5 characters')),
+        ({'max_length': 5, 'strip_whitespace': True}, '1234  ', '1234'),
         # to_upper and strip comes after pattern check
         ({'to_upper': True, 'pattern': 'abc'}, 'abc', 'ABC'),
-        ({'strip_whitespace': True, 'pattern': r'\d+$'}, 'foobar 123 ', Err("String should match pattern '\\d+$'")),
+        ({'strip_whitespace': True, 'pattern': r'\d+$'}, 'foobar 123 ', 'foobar 123'),
         ({'min_length': 1}, '🐈 Hello', '🐈 Hello'),
     ],
 )
